#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Matrix of VectorDict"""
import os, sys, inspect
cmd_folder = os.path.abspath(
    os.path.join(
        os.path.split(
            inspect.getfile( inspect.currentframe() )
        )[0] ,
        ".."
    )
)
if cmd_folder not in sys.path:
   sys.path.insert(0, cmd_folder)





from vector_dict.Clause import Clause, is_leaf, is_container

from vector_dict.Operation import  identity, mul, cast
from vector_dict.VectorDict import convert_tree
from vector_dict.SparseMatrix import SparseMatrix,Coordinates
from collections import namedtuple, defaultdict

a = convert_tree( { 'a' : { 'b' : 1 , 'c' : 2 } , 'b' : 0 } )

def title(string):
    print 
    print "*" * 80
    print string
    print "*" * 80
title( "initial dictionary" )
a.tprint()

m = SparseMatrix( 
    ## take source['a'][b'] and * -2 and put it in dst['mul']['neg2']
    ( tuple( [ 'a', 'b' ] ), tuple([ 'mul', 'neg2' ] ), mul(-2) ),
    ## guess :) 
    ( tuple([ 'a', 'c' ]), tuple([ 'mul', 'misplaced' ]), cast(float) ),
    ( tuple([ 'b' ])  , tuple([ 'a' ]) , lambda x : -4 ),
    ( tuple( [ 'a' ] ), tuple( [ 'a_dict' ] ), identity ),
    )
title( "transformed dict" )
m(a).tprint()
w = SparseMatrix()
w[ 
    Coordinates( src= tuple( [ ] ) , 
    dst =  tuple([ 'a_copy' ] ))
]=  identity 
w[ 
    Coordinates( src= tuple( [ ] ) , 
    dst =  tuple([ 'inception' ] ))
]=  m.copy()  

title( "with matrix in matrix" )
w(a).tprint()

"""
********************************************************************************
initial dictionary
********************************************************************************
{
    a = {
        c = 2,
        b = 1,
    },
    b = 0,
}

********************************************************************************
transformed dict
********************************************************************************
{
    a = -4,
    mul = {
        neg2 = -2,
    },
    a_dict = {
        c = 2,
        b = 1,
    },
}

********************************************************************************
with matrix in matrix
********************************************************************************
{
    inception = {
        a = -4,
        mul = {
            neg2 = -2,
        },
        a_dict = {
            c = 2,
            b = 1,
        },
    },
    a_copy = {
        a = {
            c = 2,
            b = 1,
        },
        b = 0,
    },
}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#WTFPL
"""test for rules of consistency in algebrae"""
#import os, sys, inspect
#cmd_folder = os.path.abspath(
#    os.path.join(
#        os.path.split(
#            inspect.getfile( inspect.currentframe() )
#        )[0] ,
#        ".."
#    )
#)
#if cmd_folder not in sys.path:
#   sys.path.insert(0, cmd_folder)



from vector_dict.VectorDict import VectorDict
from vector_dict.ConsistentAlgebrae import ConsistentAlgebrae

ConsistentAlgebrae(
    context="test",
    neutral=VectorDict(int, {}),
    one=VectorDict(int, {"one": 1, "one_and_two": 3}),
    other=VectorDict(int, {"one_and_two": - 1, "two": 2}),
    another=VectorDict(int, {"one": 3, 'two':  2, "three": 1}),
    collect_values=lambda x: x.values(),
    )





#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Clauses for common search on tree"""

import re
import types
class Clause(object):
    def __call__(self, value, *a , **kw):
        if self.__arg or self.__opt:
            real_clause = self.__clause(*self.__arg,**self.__opt) 
        else:
            real_clause = self.__clause
        return real_clause(value, *a, **kw)

    def __init__(self, functor, *a, **kw):
        self.__clause = functor
        self.__arg = a or None
        self.__opt = kw or None

find_re = Clause( lambda pattern : lambda v: not re.search(pattern,str(v)) is None ) 
"""Find if something match a regular expression"""

is_container = Clause(lambda v : isinstance(v, list) or hasattr(v, "__iter__"))
"""to know if something can contain more than one value (list , iterator, ...)"""

is_leaf = Clause(lambda v : not isinstance(v, dict))
"""is_leaf if value is not instance of dict"""

has_type = Clause(lambda _type : lambda v :  isinstance(v,_type))
"""Check if value has type _type"""

is_function = has_type( types.FunctionType )
"""check if value is a function"""

and_ = Clause( lambda v, *clauses:  all( [ clause(v) for clause in clauses ] ) )

has_all = Clause(
    lambda *k : lambda v : is_container(v) and all( map( lambda key: key in v, k))) 

or_ = Clause( lambda v, *clauses : any( [ clause(v) for clause in clauses ] )  )
not_ = Clause( lambda v, clause : not(clause(v) )  )

has_tree = Clause( lambda tree_clause: lambda v : ( not is_leaf(v) ) and v.match_tree(tree_clause)) 
"""Check if value has the subtree included in it"""

anything= Clause( lambda v : True )
"""matches anything"""





#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Dict on steroids """
from collections import namedtuple, defaultdict
from collections import Sequence, Mapping
from math import sqrt
import types
from  Clause import Clause, is_leaf, is_container, is_function
#WTFPL

__all__ = ['cos', 'dot',  'iter_object' , 'tree_from_path', 'Path',
    'flattening', 'can_be_walked', 'Element']



def convert_tree(a_tree):
    """
    convert from any other nested object to a VectorDict 
    espaecially usefull for constructing a vector dict from 
    intricated dict of dicts. 
    *Dont work as a class method (why ?)*

 >>> convert_tree({ 'a' : { 'a' : { 'r' : "yop", 'b' : { 'c' :  1 }}}}).tprint()
 {
     a : {
         a : {
             r : 'yop',
             b : {
                 c : 1,
             },
         },
     },
 }


    """
    a_vector_dict = VectorDict()

    for e in iter_object_nl(a_tree, flatten = True):
        a_vector_dict += tree_from_path( *e )
    return a_vector_dict

Element = namedtuple('Element' , "path value")

class Path(tuple):
    def __init__(self, a_tuple):
        """construct a Path from a tuple or a list 
        
 >>> p = Path( [  'a', 'b', 'c' ] )
 >>> p
 >>> ( 'a', 'b', 'c' )
 >>> p = Path( (  'a', 'b', 'c' ) )
 >>> p
 >>> ( 'a', 'b', 'c' )
        """
        self += tuple(a_tuple)

    def endswith( self, *a_tuple ):
        """check if path ends with the consecutive given has argumenbts value
 
 >>> p = Path( [ 'a', 'b', 'c' ] )
 >>> p.endswith( 'b', 'c' )
 >>> True
 >>> p.endswith( 'c', 'b' )
 >>> False
        """
        return self[len(self) - len(a_tuple) : ] == a_tuple

    def startswith( self, *a_tuple ):
        """checks if a path starts with the value 

 >>> p = Path( [ 'a', 'b', 'c', 'd' ] )
 >>> p.startswith( 'a', 'b' )
 >>> True
        """
        return self[: len( a_tuple ) ] == a_tuple

    def _contains( self, a_tuple, _from = 0, follow = 0):
        if len( a_tuple) == follow:
            return True
        index = False
        try:
            here = self[ _from:]
            if len(here ) < len(a_tuple[follow:]):
                return False
        except IndexError:
            return False

        try:
            index = here.index(a_tuple[follow] )
            return  self._contains(
                    a_tuple, 
                        index +  1 , 
                        follow + 1
                    )
        except ValueError:
            return False
        except IndexError:
            return self._contains(a_tuple, _from +  1 )
        return False

    def contains(self, *a_tuple ):
       """checks if the serie of keys is contained in a path
 
 >>> p = Path( [ 'a', 'b', 'c', 'd' ] )
 >>> p.contains( 'b', 'c' )
 >>> True

       """
       return self._contains( a_tuple )

def dot( obj1, obj2):
    """for ease of reading and writing
    equivalent to obj1.dot( obj2 )
    does the leaf by leaf product of the imbricated dict for all 
    the keys that are similar. 

 >>> dot(
        VectorDict( int, dict( x=1 , y=1, z=0) ),
        VectorDict( int, dict( x=1 , y=1, z=1) ),
     ) 
 2.0
    
    """
    return obj1.dot(obj2)

def cos( obj1, obj2):
    """for ease of reading and writing
    equivalent to the cosinus similarity obj1.cos(2)
    returns the cosinus similarity of two vectorDict 
    
 >>> cos(
        VectorDict( int, dict( x=1 , y=1) ),
        VectorDict( int, dict( x=1 , y=0) ),
     ) 
 0.7071067811865475
    """
    return obj1.cos(obj2)



def tree_from_path( *path ):
    """creating a dict from a path

 >>> tree_from_path( 'a', 'b', 'c', 1  ).tprint()
 {
     a : {
         b : {
             c : 1,
         },
     },
 }

    """
    path_to_key = list(path)
    root = VectorDict( VectorDict, { path_to_key.pop() : path_to_key.pop()})
    current = root
    while len(path_to_key):
        current = VectorDict( VectorDict, { path_to_key.pop() : current})
    return current

def can_be_walked(stuff):
    """tells if it is walkable """
    return isinstance(stuff, list) or hasattr(stuff, "__iter__")

def is_generator(stuff):
    """tells if it is a generator"""
    return isinstance(stuff, types.GeneratorType)

def flattening(a_duck, taxonomy = can_be_walked ):
    """flattening stuff // adapted from python cookbook"""
    if not taxonomy(a_duck):
        yield a_duck
    else:
        for duckling in a_duck:
            if taxonomy(duckling) and not isinstance( duckling, str) :
                for duck_eggs in flattening(duckling, taxonomy = taxonomy):
                    yield duck_eggs
            else:
                yield duckling



def iter_object_nl(obj, path=(), **opt):
    """
    Generator on all leaves of the object, return an array of the path and the
    leaf

    source:
     http://tech.blog.aknin.name/2011/12/11/walking-python-objects-recursively/
    """

    if isinstance( obj, Mapping ):
        if len(obj):
       
            for key, value in obj.iteritems():
                for child in iter_object_nl(value, path + (key,), **opt):
                    yield  child
        else:
            yield list(path) + [ obj ]
            
    else:
        if opt.get("flatten"):
            yield [ x for x in flattening( path)  ] + [ obj ]
        else:
            yield (path, obj)

def iter_object(obj, path=(), **opt):
    """
    Generator on all leaves of the object, return an array of the path and the
    leaf

    source:
     http://tech.blog.aknin.name/2011/12/11/walking-python-objects-recursively/
    """

    if isinstance(obj, Mapping):
        if len(obj):
            for key, value in obj.iteritems():
                for child in iter_object(value, path + (key,), **opt):
                    yield  child
        else:
            yield (path, obj )

    else:
        yield  opt.get("flatten") and [ 
                x for x in flattening( ( path, obj) ) 
            ] or ( path, obj )

class VectorDict(defaultdict):
    """slightly enhanced Dict"""
    def __init__(self, *a , **a_dict ):
        """Constructs like a collections.defaultdict (put sphinx ref)"""
        
        super( VectorDict, self).__init__( *a,**a_dict)

    def from_tree( self, a_tree):
        """Create a VectorDict Intrication from a tree
        Drawback there is no factory specified
        """
        self =  convert_tree( a_tree)

    def match_tree(self, a_tree):
        """does the tree given has an argument match the actual 
        tree.
 if tree leaves are Clauses, the match_tree will apply the clauses.

 **Direct Match**

 >>> a_tree = dict( 
            b = dict( 
                c = 3.0, 
                d = dict( e = True)
            ), 
        )
 >>> complex_dict = convert_tree( a_tree )
 >>> complex_dict.match_tree( dict( c= 3.0 ,d = dict( e = True) ))
 >>> False
 >>> complex_dict.match_tree( dict( b=dict( c=3.0 ,d = dict( e = True) ))
 >>> True

 **Match with clauses**

 >>> from vector_dict.Clause import Clause, anything
 >>> complex_dict.match_tree( dict( b=dict( c=3.0 , d=anything ) ) )
 >>> True
 >>> complex_dict.match_tree({ 'b': { 
    'c': Clause( lambda v : 1 < v < 5), 
    'd' : anything 
 } } )
 >>> True
 
        """

        match_to_find = len(a_tree.keys())
        if not set(a_tree).issubset( set(self.keys())):
            return False
        for k,v in a_tree.iteritems():
            if k in self:
                if is_leaf(v):
                #terminaison of the comparison tree
                    if isinstance(
                            v, Clause
                        ) or isinstance( 
                            v , types.FunctionType
                        ):
                    # if it is a clause apply it to the targeted tree
                        match_to_find -= v(self.get(k)) and 1 or 0
                    else:
                    ## it is not a clause
                        val = self.get(k)
                        if  is_leaf(val):
                        ## terminating values match
                            match_to_find -= v ==  val and 1 or 0
                        else:
                        ## the compared tree goes on, we match the keys
                            match_to_find -=  v in val  and 1 or 0
                else:
                    ## the comparison tree goes on
                    sub_tree = self.get(k)
                    if not is_leaf(sub_tree):
                        ## the compared tree goes on 
                        ## we recurse
                        match_to_find -= sub_tree.match_tree( v ) and 1 or 0
                    else:
                        ### the compared tree is smaller than the comparison 
                        ## tree
                        match_to_find -= v ==sub_tree and 1 or 0 
                        #return False
        return 0 == match_to_find


    def build_path( self, *path):
        """ implementation of constructing a path in a tree, argument is 
        a serie of key
 
 >>> a = VectorDict(int, {})
 >>> a.build_path( 'k', 1 )
 >>> a.tprint()
 {
     k = 1,
 }
 >>> a.build_path( 'b', 'n', [ 1 ] )
 >>> a.build_path( 'd', 'e',  2  )
 >>> a.build_path( 'd', 'f',  4  )
 >>> a.tprint()
 {
     k : 1,
     b : {
         n : [1],
     },
     d : {
         e : 2,
         f : 4,
     },
 }

 """
        if len(path) == 2:
            key, value = path[0:2]
            if  key in self.keys() or self.get(key):
                raise ValueError( "collision of values")
            self.__setitem__( key, value )
        if len(path) > 2:
            key, value = path[0:2]
            if key in self.keys() and self.get(key):
                if  value in self[key].keys():
                    self[key].build_path( path[1:])
            else:
                if key in self.keys():
                    raise Exception("Path already present")
            if not is_leaf( self[key] ):
                self[key] +=  tree_from_path( *path[1:] )
            else:
                self.__setitem__(key,  tree_from_path( *path[1:] ))

    def prune(self, *path):
        """delete all items at path 
        
 >>> a  = VectorDict(int, {})
 >>> a.build_path( 'g', 'todel' , True )
 >>> a.build_path( 'g', 'tokeep' , True )
 >>> a.tprint()
 {
     g : {
         tokeep : True,
         todel : True,
     },
 }
 >>> a.prune( 'g', 'todel' )
 >>> a.tprint()
 {
     g : {
         tokeep : True,
     },
 }

"""

        todel = None
        if len(path)>1:
            self.at(path[:-1]).__delitem__(path[-1])
        else:
            self.__delitem__( path[0] )


    def get_at(self, *path):
        """Get a copy of an element at the coordinates given by the path
Throw a KeyError excpetion if the path does not led to an element

 >>> from vector_dict.VectorDict import convert_tree, VectorDict
 >>> intricated = convert_tree( { 'a' : { 'a' : { 'b' : { 'c' :  1 } } } } )
 >>> intricated.get_at( 'a', 'a', 'b' )
 defaultdict(<class 'vector_dict.VectorDict.VectorDict'>, {'c': 1})
 >>> intricated.get_at( 'a', 'a', 'b', 'c' )
 1
 >>> intricated.get_at( 'oops' )
 Traceback (most recent call last):
   File "<input>", line 1, in <module>
   File "vector_dict/VectorDict.py", line 304, in get_at
     return self.at( path, None , True)
   File "vector_dict/VectorDict.py", line 330, in at
     value = here[path[ -1 ] ]
 KeyError: 'oops'
 
 """
        return self.at( path, None , True)

    def at(self, path, apply_here = None, copy = False):
        """
        gets to the mentioned path eventually apply a lambda on the value
        and return the node, 
        and copy it if mentioned. 

 >>> intricated = convert_tree( { 'a' : { 'a' : { 'b' : { 'c' :  1 } } } } )
 >>> pt = intricated.at( ( 'a', 'a', 'b' ) )
 >>> pt
 defaultdict(<class 'vector_dict.VectorDict.VectorDict'>, {'c': 1})
 >>> pt['c'] = 2
 >>> intricated.tprint()
 {
    a : {
        a : {
            b : {
                c : 2,
            },
        },
    },
 }
 >>> intricated.at( ( 'a', 'a', 'b' ), lambda x : x * -2  )
 defaultdict(<class 'vector_dict.VectorDict.VectorDict'>, {'c': -4})
 >>> intricated.pprint()
 a->a->b->c = -4

        """
        here = self
        if apply_here and not( is_function(apply_here) ):
            raise Exception("second argument must be a function to apply on path")
        if not len(path):
            if apply_here:
                raise Exception("cant apply a function on root of %r" % self)
            if copy:
                return  self.copy() 
            return   self
        for e in path[:-1]:
            if not hasattr(here, "__getitem__"):
                raise IndexError("this path dont exists")
            if not here.__getitem__(e):
                raise Exception("Path %r does not exists in the tree" %path ) 
            here = here.__getitem__(e)
        if not apply_here is None:
            here.__setitem__(path[-1],apply_here(here[path[-1]]))
        value = here[path[ -1 ] ]

        if copy and is_container(value):
            return here[path[-1]].copy()
        return value


    def __flatten_generator(func):
        def wrap(*a, **kw):
            return flattening( func( *a, **kw ), taxonomy = is_generator ) 
        wrap.__doc__ = func.__doc__
        return wrap

    #@flatten_generator
    def diff(self, other, diff_mine = None,diff_other=None, path = []):
        """Still In Development. 
        trying to have a tree dif"""

        def prune( key, adict):
            def todo():
                adict.__delitem__(key)
            return [ todo ]

        def cp_if_dict(val):
            return isinstance( v, dict) and v.copy() or v
        if not diff_mine:
            diff_mine = VectorDict( VectorDict, {} )
            diff_other = VectorDict( VectorDict, {} )
        for k, v in self.iteritems():
            if not k  in other.keys():
            ## pour que l'autre me ressemble enlevons toutes les clés
                diff_other += tree_from_path(  * [ path + [k, prune( k, self) ]  ] )
            else:
                ## k in other.keys()
                ## les clés sont dans les deux arbres
                if v != other[k] :
                    if isinstance( v , VectorDict) and isinstance( 
                        other[k], VectorDict):
                        vd_m, vd_o = v.diff(  other[k],  diff_mine, diff_other,  path + [ k ] )
                        diff_mine = diff_mine + vd_m
                        diff_other = diff_other + vd_o
                    elif isinstance( v , VectorDict) or isinstance( 
                        other[k], VectorDict):
                        diff_mine +=  tree_from_path( *[ path + [k, cp_if_dict( other[k]) ] ] )
                    else: 
                        diff_mine +=  tree_from_path( *[  path + [k, - other[k]   ] ] )

                
                else:
                    if isinstance( v, VectorDict ):
                        vd_m, vd_o = v.diff(  other[k],  diff_mine, diff_other,  path + [ k ] )
                        diff_mine = diff_mine + vd_m
                        diff_other = diff_other + vd_o
                    
        for k , v in other.iteritems():
            if not k  in self.keys():
                diff_mine += tree_from_path( *[  path + [k, v ] ] )

        return diff_mine, diff_other
               


   
    @__flatten_generator
    def find(self, predicate_on_path_value, path = tuple() ):
        """apply a fonction on value if predicate on key is found"""
        path = Path( path + tuple() )
        if predicate_on_path_value(Path( path) , self):
            yield  Element( Path( path + tuple() )  ,self )
        for k,v in self.iteritems():
            if isinstance(v, VectorDict ):
                yield  v.find( predicate_on_path_value, Path( path + ( k,)  ) ) 
            else:
                if predicate_on_path_value(Path( path + (k,) ), v):
                    yield  Element( path =  Path( path + (k,))  ,value = v  )

    def  __rdiv__(self, other):
        """reverse division"""
        return self.__div__(other)

    def __idiv__(self, other):
        """immediate division /= """
        return self.__opfactory__(other, False, lambda x,y: x/y , "__internal_divide__" )

    def __div__(self, other):
        """div with copy"""
        return self.__opfactory__(other, True, lambda x,y: x/y , "__internal_divide__" )

    def __rmul__(self, other):
        return self.__mul__(other)

    def __imul__(self, other):
        return self.__opfactory__(other, False)

    def __mul__(self, other):
        return self.__opfactory__(other, True)

    def __opfactory__(
            self, 
            other, 
            copy = True, 
            extern_operation = lambda x,y : x*y ,
            intern_operation = "__internal_mul__"
        ):
        """muler"""
        ## __imul__ later
        if not isinstance( self, VectorDict ):
            self, other= other, self
        a_copy = copy and self.copy() or self

        if not isinstance(other, VectorDict):
            for k, v in self.iteritems():
                a_copy[k] = extern_operation( v , other)
            return a_copy
        else:
            return getattr(a_copy, intern_operation)(other)

    def __neg__(self):
        for k, v in self.iteritems():
           self[k] = -1 * v
        return self

    
    def as_vector_iter(self, path=()):
        """
        iterator on key value pair of nested dict in the form of 
        set( key0, key1, key2 ), child
        for a dict, therefore making a n-depth dict being homomorph
        to a single dimension vector in the form of 
        k , v 
        where k is the path, v is the leaf value
        source:
        http://tech.blog.aknin.name/2011/12/11/walking-python-objects-recursively/

 >>> a = convert_tree({ 'a' : { 'a' : { 'r' : "yop" , 'b' : { 'c' :  1 }}}})
 >>> a.tprint()
 {
     a = {
         a = {
             r = 'yop',
             b = {
                 c = 1,
             },
         },
     },
 }
 >>> [ e for e in a.as_vector_iter() ]
 [(('a', 'a', 'r'), 'yop'), (('a', 'a', 'b', 'c'), 1)]
        """
        return iter_object(self,(),flatten=False)
    
    def as_row_iter(self, path=(), **arg): 
        """
        iterator on key value pair of nested dict yielding items in the form
        set( key0, key1, key2 , child)
        very useful for turning a dict in a row for a csv output
        all keys and values are flattened

 >>> a = convert_tree({ 'a' : { 'a' : { 'r' : "yop" , 'b' : { 'c' :  1 }}}})
 >>> a.tprint()
 {
     a : {
         a : {
             r : 'yop',
             b : {
                 c : 1,
             },
         },
     },
 }
 >>> [ e for e in a.as_row_iter() ]
 [['a', 'a', 'r', 'yop'], ['a', 'a', 'b', 'c', 1]]


        """
        arg["flatten"] = arg.get("flatten", True)
        return iter_object(self,(),**arg )

    def __not__(self):
        """ not on all leaves of the dict

 >>> from vector_dict.VectorDict import convert_tree, VectorDict,Element,Path
 >>> a = convert_tree(dict(a = dict(tt=True, tf=True, ft=False, ff=False), c= False))
 >>> a.__not__().tprint()
 {
     a : {
         tf : False,
         tt : False,
         ft : True,
         ff : True,
     },
     c : True,
 }
 >>> a.tprint()
 {
     a : {
         tf : True,
         tt : True,
         ft : False,
         ff : False,
     },
     c : False,
 }

"""

        new_dict=self.copy()
        for k, v in new_dict.iteritems():
            if not isinstance(v, VectorDict):
                new_dict[k] = not v
            else:
                new_dict[k] = v.__not__()
        return new_dict


    def __or__(self,other):
        """or on all leaves

 >>> b = convert_tree(dict(a = dict(tt=True, tf=False, ft=True, ff=False), b= True))
 >>> a = convert_tree(dict(a = dict(tt=True, tf=True, ft=False, ff=False), c= False))
 >>> ( b | a).tprint()
 {
     a : {
         tf : True,
         tt : True,
         ft : True,
         ff : False,
     },
     c : False,
     b : True,
 }
"""

        new_dict = self.copy()
        common_key = set( self.keys() ) & set(other.keys() )
        for k in common_key:
                new_dict[k] = (self[k]).__or__( other[k] )
        for k in set( other.keys()) - common_key:
            new_dict[k] =  other[k] 
            
        return new_dict

    def __and__(self,other):
        """and on all leaves

 >>> b = convert_tree(dict(a = dict(tt=True, tf=False, ft=True, ff=False), b= True))
 >>> a = convert_tree(dict(a = dict(tt=True, tf=True, ft=False, ff=False), c= False))
 >>> ( b & a).tprint()
 {
     a : {
         tf : False,
         tt : True,
         ft : False,
         ff : False,
     },
 }
"""

        common_key =  set( self.keys() ) &  set( other.keys() )
        new_dict = VectorDict(VectorDict, dict() )
        for k in common_key:
                new_dict[k] = (self[k]).__and__( other[k] )
        return new_dict

    def __xor__(self,other):
        """xor on all leaves 
 >>> b = convert_tree(dict(a = dict(tt=True, tf=False, ft=True, ff=False), b= True))
 >>> a = convert_tree(dict(a = dict(tt=True, tf=True, ft=False, ff=False), c= False))
 >>> ( b ^ a).tprint()
 {
     'a' : {
         'tf' : False,
         'tt' : True,
         'ft' : False,
         'ff' : True,
     },
 }
"""


        return (  self.__not__() & other ) | ( other.__not__() & self )

    def intersection( self, other):
        """Return all elements common in two different trees
        raise an exception if both leaves are different

        #TOFIX : dont make a newdict if doing in place operations


 >>> from vector_dict.VectorDict import convert_tree, VectorDict,Element,Path
 >>> a = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, c = 2  ) ) } )
 >>> b = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, d = 1  ) ) } )
 >>> a.intersection(b).tprint()
 {
     a : {
         b : 1,
     },
 }
 >>> b = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, c = 1  ) ) } )
 >>> a.intersection(b).tprint()
 Traceback (most recent call last):
   File "<input>", line 1, in <module>
   File "vector_dict/VectorDict.py", line 634, in intersection
     new_dict[k] = (self[k]).intersection( other[k] )
   File "vector_dict/VectorDict.py", line 639, in intersection
     other[k]
 Exception: ('CollisionError', '2 != 1')
 

"""

        common_key =  set( self.keys() ) &  set( other.keys() )
        new_dict = VectorDict(VectorDict, dict() )
        for k in common_key:
        ## and what about sets ? 
        ## try a key or value made of a forzen set
            if  hasattr( self[k], "intersection") :
                new_dict[k] = (self[k]).intersection( other[k] )
            else:
                if self[k] != other[k]:
                    raise Exception("CollisionError","%s != %s" % (  
                            self[k] ,
                            other[k]
                        ) )
                new_dict[k] = self[k]
        return new_dict or VectorDict()
    
    def union(self, other, intersection=None):
        """return the union of two dicts

 >>> from vector_dict.VectorDict import convert_tree, VectorDict,Element,Path
 >>> b = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, c = 1  ) ) } )
 >>> a = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, d = 2, c=1  ) ), 'e' :  1 } )
 >>> b.union(a)
 defaultdict(<type 'int'>, {'a': defaultdict(<type 'int'>, {'c': 1, 'b': 1, 'd': 2}), 'e': 1})
 >>> a = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, d = 2, c=3  ) ) } )
 >>> b.union(a)
 Traceback (most recent call last):
   File "<input>", line 1, in <module>
   File "vector_dict/VectorDict.py", line 669, in union
     return self + other - self.intersection(other)
   File "vector_dict/VectorDict.py", line 658, in intersection
     new_dict[k] = (self[k]).intersection( other[k] )
   File "vector_dict/VectorDict.py", line 663, in intersection
     other[k]
 Exception: ('CollisionError', '1 != 3')
 

"""
    ###Dont work
        #return - self.intersection(other) + self + other
        ## take  intersection 
        union  = intersection or self.intersection(other)
        ## add self not in intersection
        ### easy case

        common_keys = set( union.keys() ) 
        orthogonal_keys_self = set(self).symmetric_difference( set( union.keys() ))
        orthogonal_keys_other = set(other).symmetric_difference( set( union.keys() ) )


        for k in orthogonal_keys_self:
            if not is_leaf(self[k]):
                union[k] += self[k]
            else:
               raise( Exception("Wtf?") )
        
        for k in orthogonal_keys_other:
            if not is_leaf(other[k]):
                union[k] += other[k]

        for k in common_keys:
            if not self[k] == other[k]:
                if isinstance(union[k], VectorDict) :
                    union[k] = self[k].union(other[k], intersection and intersection.get(k) or VectorDict()  )
                else: 
                    union[k] = self[k].union(other[k])
        return union
    
    def symmetric_difference( self, other):
        """ return elements present only elements in one of the dict
        Throw a Collision Error if two leaves in each tree are different

 >>> from vector_dict.VectorDict import convert_tree, VectorDict,Element,Path
 >>> a = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, d = 2, c=1  ) ), 'e' :  1 } )
 >>> b = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, c = 1   ) ) } )
 >>> a.symmetric_difference(b)
 defaultdict(<type 'int'>, {'a': defaultdict(<type 'int'>, {'d': 2}), 'e': 1})
 >>> b = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, c = 2   ) ) } )
 >>> a.symmetric_difference(b)
 Traceback (most recent call last):
   File "<input>", line 1, in <module>
   File "vector_dict/VectorDict.py", line 694, in symmetric_difference
     for path,v in self.intersection(other).as_vector_iter():
   File "vector_dict/VectorDict.py", line 658, in intersection
     new_dict[k] = (self[k]).intersection( other[k] )
   File "vector_dict/VectorDict.py", line 663, in intersection
     other[k]
 Exception: ('CollisionError', '1 != 2')


"""
        new_dict = self + other
        for path,v in self.intersection(other).as_vector_iter():
            new_dict.prune( *path )
        return new_dict

    def issubset( self, other ):
        """tells if all element of self are included in other

 >>> from vector_dict.VectorDict import convert_tree, VectorDict,Element,Path
 >>> a = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, d = 2, c=1  ) ), 'e' :  1 } )
 >>> b = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, c = 1   ) ) } )
 >>> b.issubset(a)
 True
 >>> a.issubset(b)
 False

"""
        return self.intersection(other) == self

    def issuperset(self, other):
        """tells if all element of other is included in self
throws an exception if two leaves in the two trees have different 
values

 >>> from vector_dict.VectorDict import convert_tree, VectorDict,Element,Path
 >>> a = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, d = 2, c=1  ) ), 'e' :  1 } )
 >>> b = VectorDict( int, { 'a' : VectorDict( int, dict( b = 1, c = 1   ) ) } )
 >>> a.issuperset(b)
 True
 >>> b.issuperset(a)
 False

"""

        return self.union(other) == self

    def __internal_divide__(self, other):
        """dividing two vectors internaly"""

        common_key =  set( self.keys() ) &  set( other.keys() )
        ## how could I get the factory of the default dict ? 
        new_dict = VectorDict(VectorDict, dict() )
        for k in common_key:
            if  hasattr( self[k], "__internal_divide__") :
                new_dict[k] = (self[k]).__internal_divide__( other[k] )
            else:
                new_dict[k] = self[k] / other[k]
        return new_dict

    def __internal_union__(self, other, copy = True):
        """what to do when you do an union on two dicts ? """
        common_key =  set( self.keys() ) &  set( other.keys() )
        new_dict = copy and VectorDict( None, {} ) or self
        for k in common_key:
            if  hasattr( other[k], "__internal_union__") :
                new_dict[k] = (self[k]).__internal_union__( other[k] )
            else:
                new_dict[k] = self[k] * other[k]

    def __internal_mul__(self, other):
        """multiplying to vectors as one vector of homothetia * vector
        it is a shortcut for a multiplication of a diagonal matrix
        missing keys in the pseudo diagonal matrix are pruned"""
        
        common_key =  set( self.keys() ) &  set( other.keys() )
        ## how could I get the factory of the default dict ? 
        ## hum hum, and what about in place modification ? 
        new_dict = VectorDict(VectorDict, dict() )
        for k in common_key:
            if  hasattr( self[k], "__internal_mul__") :
                new_dict[k] = (self[k]).__internal_mul__( other[k] )
            else:
                new_dict[k] = self[k] * other[k]
        return new_dict
    
    def dot(self, other):
        """scalar  = sum items self * other for each distinct key in common
        norm of the projection of self on other"""
        return sum( [ 1.0 * v for k,v in ( self * other ).as_vector_iter()  ] )
    
    #def values(self):
    #    return ( v for k,v in self.as_vector_iter() )

    def norm(self):
        """norm of a vector dict = sqrt( a . a  )"""
        return sqrt(self.dot(self))

    def cos( self, other ):
        """cosine similarity of two vector dicts
        a . b / ( ||a||*||b|| )
        """
        return 1.0 * self.dot( other) / self.norm() / other.norm()

    def jaccard(self, other):
        """jaccard similariry of two vectors dicts
        a . b  / ( ||a||^2 + ||b||^2 - a . b )
        """
        return 1.0 * self.dot(other) / (
            float( self.norm()) ** 2 +
            float( other.norm()) ** 2-
            self.dot(other) 
        )


    def __sub__(self, other):
        """subber"""
        positive = self.copy()
        negative = other
        neg_key = set(negative.keys())
        pos_key = set(positive.keys())
        for k, v in negative.iteritems():
            if k in positive.keys():
                positive[k] -= v
            else:
                positive[k] = -v
        return positive



    def tformat(self, indent_level = 0, base_indent = 4):
        """pretty printing in a tree like form a la Perl"""
        offset = " " *  indent_level * base_indent
        toreturn = '{\n'
        for k,v in self.iteritems():
            toreturn += offset + ( " " * base_indent ) +  '%s : ' %( 
                repr(k) or '<BUG>')
            if hasattr( v, "tformat"):
                toreturn += v.tformat( indent_level+1, base_indent )
            else:
                toreturn += "%s" % ( isinstance( v, ( str, unicode) ) and ( "'%s'" % v ) or repr(v) )
            toreturn += ',\n'
        toreturn += offset +  '}'
        return toreturn 

    def tprint( self, indent_level = 0, base_indent = 4):
        """pretty printing with indentation in tradiotionnal fashion"""
        print self.tformat( indent_level, base_indent )

    def pformat(self):
        return  "\n".join( [ 
                    "%s = %s" % (
                        "->".join( map(unicode, x[0])), 
                       isinstance( x[1], ( str, unicode) ) and ( "'%s'" % x[1] ) or repr(x[1])  
                    ) for x in self.as_vector_iter() ] )
    def pprint(self):
        """ pretty printing the VectorDict in flattened vectorish representation"""
        print self.pformat() 
                
    def __add__(left1, left2):
        """adder"""
        left1_big = len(left1.keys()) > len(left2.keys())
        bigger = left1_big and left1.copy() or left2.copy()
        smaller = left1_big and left2 or left1
        for k, v in smaller.iteritems():
            if k in bigger.keys():
                bigger.__setitem__(k, bigger[k] + v )
            else:
                bigger.__setitem__(k,  v)
        return bigger
    
    def __iadd__(self, other):
#        print "self %r" % self
#        self = self + other
        #print "other %r" % other
        for k, v in other.iteritems():
            if k in self.keys():
                self[k] +=  v
            else:
                self.__setitem__(k, v)
        #print "self %r" % self
        return self

if '__main__' == __name__:
    print "undestranding 2 * array"
    from numpy import array, ndarray
   # def tracing(callme):
   #     def wrapper(callme):
   #         call_my_name = callme.__name__ or "oops"
   #         call_arg = "%r / %r" % (a, kw)
   #         print "IN: %r(%r)" % (call_my_name, call_arg)
   #         try:
   #             res = callme(*a, **kw)
   #         except Exception as e:
   #             print "OUT: E  %r" %  e
   #         print "OUT: %r => %r" % (callme, res)
   #         return result
   #     return wrapper
   # for method in VectorDict.__dict__.keys():
   #     if method.startswith("__"):
   #         print "%r " % (method,)
   #         setattr(int, method, tracing(method))

    print u"testing"

    a = VectorDict(VectorDict, {'FR': VectorDict(VectorDict,
        VectorDict(array, {'paris': array([1, 3])}))})

    print u"testing"
    a += VectorDict(
            VectorDict, {
                'FR': VectorDict(
                    VectorDict, VectorDict(
                            array, {'paris': array([1, 3])}
                       )
                   )
           }
       )
    b = VectorDict(
            VectorDict, {
                'FR':
                 VectorDict(VectorDict,
                        VectorDict(
                            array, {'lyon': array([1, 3])}
                       )
                    )
               }
          )
    print "%r" % a
    print "****%r" % b
    b.pprint()
    a += b
    for el in iter_object(a):
        print "<%r>" % (el or "",)
    print "%r" % "-".join( [ repr(k) for k,v in a.as_vector_iter() ] )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""misc function that can be used on tree or leaf
just for convenience

identity : 
    return itself

copy : 
    return a copy of a value

mul(by) : 
    return a function that multiples by a value

cast(type) : 
    cast the element in given type
"""

identity = lambda x : x 
copy = lambda x : x.copy()
mul = lambda by :  lambda x : x * by 
cast = lambda _type : lambda x : _type(x)



# t!/usr/bin/python
# -*- coding: utf-8 -*-
#WTFPL
"""Trying lame cosinus on text
to create the mysoginie and troll input text file
you need *nix 
fortunes / fortunes-fr
and then 
fortune mysoginie -m '' | egrep -v -r "^%" | egrep -v  '\-\+\-' > myso-fr.txt 
fortune tribune-linuxfr  -m '' | egrep -v -r "^%" | egrep -v  '\-\+\-' >  troll.txt

chienne is the text of a feminist site http://www.chiennesdegarde.com/

blague miso is  a random mysoginistic joke on internet. 

The higher a cos is the more two things are similar. 

I used pydot to make a svg map of the path / frequency, just for fun

"""


import os, sys, inspect
cmd_folder = os.path.abspath(
    os.path.join(
        os.path.split(
            inspect.getfile( inspect.currentframe() )
        )[0] ,
        ".."
    )
)
if cmd_folder not in sys.path:
   sys.path.insert(0, cmd_folder)




import re
from vector_dict.VectorDict import VectorDict, tree_from_path, flattening
import pydot as pd
from codecs import open as open

def dot_graph( dict_graph, out_name, gopts = dict( fontsize = 20.0, size = "100.0,100.0", dpi= 1000.0 )):
    graph = pd.Dot(graph_type='digraph', **gopts )
    node = set()
    edge = []
    label = [] 
    for pair in dict_graph.as_row_iter(flatten=False):
        path = [ "ROOT" ] +  list(pair[0]) + list( [ pair[1] ] )  
        ## on vire weigth
        path.pop(-2) 
        node = node | set( x for x in flattening(path)  )
        
        label += [ u"->".join(unicode(x) for x in path[1::] ) ]

        while len(path) >= 2:
            if isinstance( path[1], int) or ( path[0:2]) not in edge :
                edge += [ ( path[0:2] ) ]
            path.pop(0)
    for n in node:
        graph.add_node( pd.Node( n) )
    for e in edge:
        arg = dict()
        if isinstance(e[1],int):
            arg = dict( label = label.pop(0) )
        graph.add_edge( pd.Edge( e[0], e[1], arrowhead="normal", **arg ))
    graph.write_jpeg(out_name)

t1 = u"""Je vais à la pêche aux moules moules moules, qui viendras avec moi?"""
t2 = u"""je vais à la pêche électorale aux voies"""
t3 = u"""tous les chemins mènent à Rome"""
t4 = u"""Le je du jeu, Jeune je vois, est il une jeunesse ?"""
t5 = u"""je jeune à jeun, jeu à jouer"""

def path_collider(vector, path):
    """adding a path to a vector"""
    path_copy = list(path)
    if not isinstance( vector, VectorDict):
        if len(vector):
            vector = tree_from_path( * ( vector + [ "weight" , 1 ] ))
        else:
            vector = VectorDict( VectorDict, dict() )
    current = vector
    while len(path) :
        current_key = path.pop(0)
        if current_key in current.keys():
            
            current = current[current_key]
            if current.get("weight", False):
                current["weight"]+=1
        else:
            path = [ current_key ] + path
            break
    if len(path):
        vector.add_path( path_copy + [ "weight" , 1 ] )
    return vector 

def text_grapher(unicode_text):
    sp_pattern = re.compile( "[\s\-\,\']+", re.M)
    return reduce( 
        path_collider,  
        map( 
                lambda string : filter(unicode.isalpha,list(string)) ,
                map( unicode.lower,sp_pattern.split(unicode_text ) ) 
       )
    )
print "sample of a text to word counter"
text_grapher(t1).pprint()

print "some cos"
print "cos t1, t2"
print text_grapher(t1).cos(text_grapher(t2))
print "cos t1, t3"
print text_grapher(t1).cos(text_grapher(t3))

print "cos t4, t5"
print text_grapher(t4).cos(text_grapher(t5))


mysogine = text_grapher( open("myso-fr.txt","rt",encoding= "utf-8" ).read() )
mysogine.pprint()
dot_graph( mysogine, "mysogine.svg")


troll = text_grapher(  open("troll.txt","rt", encoding='utf-8').read() )

chienne = text_grapher(  open("chienne.txt","rt", encoding='utf-8').read() )

print "la page de garde de chienne de garde est elle troll ou missogyne? %r ou %r " % (troll.cos( chienne) , mysogine.cos( chienne))
blague = text_grapher(  open("blague-myso.txt","rt", encoding='utf-8').read() )
print "blague mysogyne est elle un troll ou misogyne? %r ou %r " % (troll.cos( blague) , mysogine.cos( blague))

print "troll cos mys = %r" % troll.cos( mysogine )



from vector_dict.VectorDict import Path, VectorDict, iter_object, tree_from_path, convert_tree
v = { 'a' : { 'a' : { 'b' : 1 } } } 
w = VectorDict()
w.add_path( [ 'a', 'a', "c", 5 ] )
# OUT: defaultdict(<class 'vector_dict.VectorDict.VectorDict'>, {'a': defaultdict(<class 'vector_dict.VectorDict.VectorDict'>, {'a': defaultdict(<class 'vector_dict.VectorDict.VectorDict'>, {'b': 1})})}) ('a', 'a', 'c', 5)
print "???"
w.pprint()
# OUT: u'a->a->b'=1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Simple test of dictionnary distance with strings being casted as
vectors of letters. And comparaison of misc candidates to the "reference"
3 measures are used : 
cos : aka vectors pointing in the same direction
projection : one vector is projected on another one, and we normalize
on the norm of the reference, the closest to one are best fits
difference : each vectors are substracted and we compare the norms

"""
import os, sys, inspect
cmd_folder = os.path.abspath(
    os.path.join(
        os.path.split(
            inspect.getfile( inspect.currentframe() )
        )[0] ,
        ".."
    )
)
if cmd_folder not in sys.path:
   sys.path.insert(0, cmd_folder)


from vector_dict.VectorDict import VectorDict


def closest_to_one(a_list, **arg):
    """return a sorted
    based on comparing values that are in absolute values 
    the closest to one"""
    return sorted(
        a_list,
        cmp = lambda x, y: cmp(
                abs(x - 1),
                abs(y - 1)
            ),
        **arg
    )


def closest( candidate ):
    """ misc test of distance"""
    cand_vect = string_as_vect(candidate)
    print "** for the word <%s>" % candidate
    
    print "better jaccard similarity : "
    print "%s/%r" % sorted(
        [ (w, cand_vect.jaccard(v)) for w, v in vectors.iteritems() ],
        key = lambda x: x[1], reverse=True
    )[0]


    print "better cos : "
    print "%s/%r" % sorted(
        [ (w, cand_vect.cos(v)) for w, v in vectors.iteritems() ],
        key = lambda x: x[1], reverse=True
    )[0]

    print "better projection relative to projected: "
    print "%s/%r" % closest_to_one(
        [ (w, cand_vect.dot(v) / ( v.norm() ** 2 ) ) 
            for w, v in vectors.iteritems() ],
        key = lambda x: x[1]
    )[0]
    
    print "better projection relative to candidate : "
    print "%s/%r" % closest_to_one(
        [ (w, cand_vect.dot(v) / ( cand_vect.norm() ** 2 ) ) 
            for w, v in vectors.iteritems() ],
        key = lambda x: x[1]
    )[0]
    print "least difference: "
    print "%s/%r" % sorted(
        [ (w, (cand_vect - v).norm() / cand_vect.norm() ) for w, v in vectors.iteritems() ],
        key = lambda x: x[1],

    )[0]
    return True


string_as_vect = lambda string :  reduce( VectorDict.__add__,
	( VectorDict(float,  { unicode(c) : 1.0 } )   for c in string )
)


could_be = [
    u"commisération",
    u"commissaire",
    u"commis",
    u"salopette",
    u"justice",
    u"singulier",
    u"ovni",
    u"sacarstique",
    u"comme",
    u"écnomie",
    u"économe",
    ]


vectors =  dict( ( noun, string_as_vect(noun) ) for noun in could_be )

map( closest, [ u"com", u"misère", u"commis", u"signifiant", u"komisaire",
    u"lol", u"mizerre", u"chromosome", u"noscomiale", u"économique" ] )




#!env python
# -*- coding: utf-8 -*-
"""
playing with cos and dot product of dict

"""
import os, sys, inspect
cmd_folder = os.path.abspath(
    os.path.join(
        os.path.split(
            inspect.getfile( inspect.currentframe() )
        )[0] ,
        ".."
    )
)
if cmd_folder not in sys.path:
   sys.path.insert(0, cmd_folder)

from math import sqrt
from numpy import array
from vector_dict.VectorDict import VectorDict


def assert_is_cos(func):
    """verify that a functions returns an int or a float in [-1;1]
    sneakily cast the return value in float to check the return value
    is castable as float
    """
    def wrapper(*a, **kw):
        res = func(*a, **kw)
        if 1 >= float(res) >= -1:
            try:
                return float(res)
            except Exception as e:
                raise Exception(
                    "wrong type %r for cos return (%r)" % (
                    res.__class, e
                    )
                )
        else:
            raise Exception("Cosinus : invalid range %r %r(%r ,%r) " % (
                    res, func, a, kw
                )
            )
        return res
    return wrapper

data = [
    dict(
        physical = dict(
            eye = "00FF00",
            heigth = 1.92,
            weigth = 52.1
        ),
        name = "jenny",
        salary = 500.00,
        presentation= [
            ("modeling show", "Paris", 2012),
            ("green cover", "Vogue", 2011),
        ],
    ),
    dict(
        physical = dict(
            eye = "00FFA0",
            heigth = 1.72,
            weigth = 48.1
        ),
        name = "noxco",
        salary = 700.00,
        presentation= [
            ("green cover", "Vogue", 2011),
            ("catholic fashion show", "Roma", 1977),
        ],
        program = ["python"],
    ),
    dict(
        physical = dict(
            eye = "000000",
            heigth = 1.65,
            weigth = 52.1
        ),
        name = "Derpina",
        salary = 300.00,
        program = ["python", "Perl"],
        hobby = ["sky", "9gag"]
    ),
]


@assert_is_cos
def bankable_cos( input_dict ):
    """the more a model has been exposed the more valuable it is"""
    return 1.0 * (  2.0 *  min( len(input_dict.get("presentation",[])), 3) / 3 - 1 )

def matrix( input_dict ):
    return VectorDict( float, 
        dict( 
            eye = eye_cos( "00FF00", input_dict ),
            fitness = imc_cos( 
                imc( 
                    input_dict.get("physical", dict() )
                ), 
                14.0, 
                5
            ),
            ## as long as its between -1, 1 it is a cos :) why bother 
            ## the more someone took parts in event (given a threshold) 
            ## then he or she is bankable
            bankable = bankable_cos( input_dict ),
            extra = extra_cos( input_dict ),
            not_too_expensive = wage_cos(300.0, 700, input_dict )
       )
    )

@assert_is_cos
def extra_cos( input_dict ):
    """discrete cos value sample : 
    1 (ideal) if has hobbies + python
    2 mixed case if has hobby and program in perl & python or
    program in python only
    3 null if orthogonal (no values in fields or projection is 0
    """
    return ( 
        ( "python" in input_dict.get("program", [] ) and 1 or -1 ) - 
        ( "Perl" in input_dict.get("program", [] ) and -1 or 0 ) + 
        ( input_dict.get("hobby") is not None and 1 or 0 )
    ) / 2
    
@assert_is_cos
def wage_cos(optimal, max_salary, input_dict ):
    """sample of float value illustrating what a cos means for float
    in regard to the utility
    1) max (1)  if you dont have to pay (salary = 0)
    2) orthogonal if not knwon or pay what you expect 
    3) (-1) min if you pay more than what you expected with a floor"""
    salary = input_dict.get("salary", max_salary )
    if salary >= max_salary:
        return -1
    if salary > optimal:
        return  (  optimal - salary ) / (  max_salary - optimal)
    return  ( salary / optimal )  - 1.0 

def scalar( v1, v2 ):
    """norm of the dot product of two vectors"""
    return (v1 ).dot(v2 ) 

def norm( v1 ):
    """ norm of a numpy array"""
    return sqrt( scalar( v1, v1 ) )

def imc( phys_dict ):
    """I really hate imc stuff, so this is a random formula"""
    weigth = 1.0 * phys_dict.get("weigth",120.0) 
    heigth = 1.0 *  phys_dict.get("heigth",1.55) 
    return  weigth * weigth / ( 100.0 * heigth  )

@assert_is_cos
def imc_cos( imc, ideal_imc, max_var):
    """return 1 if it is exactly the imc
    -1 if imc belongs to [ imc + max_var, inf [ or ] 0; imc - max_var ]
    linear in between"""
    return  max( - 2 *  abs( imc - ideal_imc )  / ( max_var ) + 1 , -1 )

@assert_is_cos 
def eye_cos( prefered_color, input_dict):
    """ a true simmilaruty cosins the way it is supposed to be done
    after we have split color in usual css format to a rgb vector
    """
    phys = input_dict.get("physical", dict())
    rgb_vect = array( [ int(x) for x in bytearray.fromhex( 
        phys.get( "eye", "000000" ) )[0::1]
    ] )
    prefered_vect =  array([ int(x) for x in bytearray.fromhex(prefered_color)[0::1] ] )
    max_size = norm( prefered_vect ) * norm( rgb_vect ) 
    return  max_size > 0  and  ( 
                1.0 * sum( prefered_vect * rgb_vect)  / ( max_size )
            ) or -1


criterion_ponderation = VectorDict(float, dict( 
    eye = 1 ,
    fitness = 5,
    bankable = 4,
    extra = .5,
    not_too_expensive = .1
) )

norm_of_choice = criterion_ponderation.dot(criterion_ponderation)


for candidate,ldot in    map ( 
            lambda x :  [ 
                VectorDict(VectorDict,x) , criterion_ponderation.dot( 
                    criterion_ponderation * matrix( x ) 
                )
            ] , 
            data
        ) : 
    if ldot > .5:
         candidate.pprint()
         print "fitting with a projection of %f" % ldot

criterion_ponderation = VectorDict(float, dict( 
    eye = 1.0,
) )
print "\n"
print "*" * 80
print "must have green eyes " 

for candidate,ldot in    map ( 
            lambda x :  [ 
                VectorDict(VectorDict,x), criterion_ponderation.dot( matrix( x ) )
            ] ,
            data
        ) : 
    #criterion_ponderation.homothetia(matrix(candidate)).pprint()
    if 1.0 == ldot:
         print "MATCH"
         candidate.pprint()
         print "has a projection on maximum fit of 1 %f" % ldot

criterion_ponderation = VectorDict(float, dict( 
    fitness = 1.0 ,
) )

by_fitness = map ( 
            lambda x :(   
                VectorDict(VectorDict,x), criterion_ponderation.dot(matrix( x ) )
                ), 
            data
    )  

print "*" * 80

print "best candidate IMC is " 
candidate , ldot = sorted( by_fitness, cmp= lambda x,y : - cmp( x[1], y[1] ))[0]
print "with a projection of %f" % ldot 
candidate.pprint()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from vector_dict.VectorDict import convert_tree
a = convert_tree( dict( a = 1, e = dict( b = dict( c= 1, d = True ) ), x= [ 1 , 2], z= 1.0  ))
### Traditionnal representation of a rooted tree made 
### when implemented with dict of dict
a.tprint()
# OUT: {
# OUT:     a = 1,
# OUT:     x = [1, 2],
# OUT:     z = 1.0,
# OUT:     e = {
# OUT:         b = {
# OUT:             c = 1,
# OUT:             d = True,
# OUT:         },
# OUT:     },
# OUT: }


### Flattening a tree as a vector
### which makes tree homeomorph to a vector
a.pprint()
# OUT: a = 1
# OUT: x = [1, 2]
# OUT: z = 1.0
# OUT: e->b->c = 1
# OUT: e->b->d = True




#!/usr/bin/python
# -*- coding: utf-8 -*-
#WTFPL
"""test for rules of consistency in algebrae
call the script as it is to see the result for various objects
(int, str, list, numpy array, VectorDict)
and rules tested. 

Mainly the tests are coming out of a math book
"""

from copy import deepcopy


from VectorDict import can_be_walked, VectorDict

def try_copy_or_copy(self, src, dst):
    copy = None
    if hasattr(src, "copy"):
        copy = src.copy()
    else:
        copy = deepcopy(src)
    setattr(self, dst, copy)


class ConsistentAlgebrae(object):
    """test wether an addition for two object is consistant"""

    def __init__(self, **kw):
        """
        only method really callable. 
        Arguments : 
            - neutral : neutral element of addition for the object ; 
            - scalar : real, float, or complex (normaly anything that is 1D, 
              and follow algebraic rules);
            - one : an element to test
            - other : other element to test
        
        
        optionnal :
            - other_scalar : real, float, or complex 
              (normaly anything that is 1D, and follow algebraic rules);
            - context : default  "print" make it verbose ;
            - collect_values : default lambda x : x, if testing for conservation
              a lambda fonction for getting the values
        
        """
        self.fail = 0
        self.context= kw.get("context", "print" )
        self.counter = 0
        self.success = 0
        self.neutral = None
        self.one = None
        self.other = None
        self.another = None
        self.algebraic_logic = False
        self._neutral = kw["neutral"]
        self._another = kw["another"]
        self._one = kw["one"]
        self._other = self._other = kw["other"]
        self.equal = kw.get("equal", None)
        self.scalar = kw.get("scalar", 3)
        self.other_scalar = kw.get("other_scalar", 4)
        self.collect_values = kw.get("collect_values", lambda x: x)

        self.pre_test()
        self.LesserCommutativity()
        if hasattr(self.one, '__rmul__'):
            self.BetterCommutativity()

        if hasattr(self.one, '__div__'):
            self.EvenBetterCommutativity()

        if can_be_walked(self.one):
            self.Conservation()

        self.finalize()
    
    def EvenBetterCommutativity(self):
        self.test_fraction_consisentcy()
        self.test_div_consisentcy()

    def pre_test(self):
        if "print" == self.context :
            print "\n" + "*" * 50
            print "\ntesting for  %r class\n" % (self._one.__class__.__name__)
            print " a = %r" % self._one
            print " b = %r" % self._other
            print " c = %r" % self._another
            print " an_int = %r " % self.scalar
            print " other_scalar = %r " % self.other_scalar
            print " neutral element for addition is %r " % self._neutral
            print "\n" + "*" * 50



    def finalize(self):
        """success or not ? """
        if "print" == self.context:
            print "*" * 50 + "\n"
            print "%(counter)r/%(success)r" % self.__dict__
            if self.counter  == self.success and self.algebraic_logic:
                print "%r respects the algebraic acceptation of addition" % (
                    self._one.__class__
                )

            else:
                print "%r follows the dutch logic " % (
                self._one.__class__
            )
            print "\n" + "*" * 50
        else:
            print "%(success)d/%(counter)d test passed" % ( self.__dict__ )
            if self.success == self.counter:
                print "test PASSED"
            else: 
                raise Exception("Test Failed")

    def fixture_and_test(method):
        def pprint(self,method, res, left, right):
            print "\ntest #%d" % self.counter
            print method.__doc__
            print "%s is %s" % (method.__name__, res)
            
            if "ko" == res:
                print res
                print "%r != %r " % (left, right)
        def praise(self,method, res, left, right):
            if "ko" == res:
                print "FAIL"
                print "test #%d" % self.counter
                print method.__doc__
                print "%r != %r " % (left, right)
                print "%r" % res
                self.fail += 1

        def reinit_me(self, *a, **kw):
            to_reinit = dict(_one="one",
                             _other="other",
                             _another="another",
                             _neutral="neutral")
            for src, dst in to_reinit.items():
                obj_src = getattr(self, src)
                try_copy_or_copy(self, obj_src, dst)

            res = "Arg : "
            is_equal = False
            (left,right) = (None , None)
            try:
                (left, right) = method(self, *a, **kw)
                if self.equal:
                    is_equal = self.equal(left, right)
                else:
                    is_equal = left == right
                res = is_equal and "ok" or "ko"
            except Exception as e:
                res += "%r" % e
            finally:
                self.counter += 1
                self.success += is_equal and 1 or 0
                if 'print' == self.context:
                    pprint(self, method, res, left, right )
                else:
                    praise(self, method, res, left, right)
        reinit_me.__doc__ = method.__doc__ 
        return reinit_me

    @fixture_and_test
    def test_commutativity(self):
        """ a + b = b + a """
        return (self.other + self.one, self.one + self.other)

    @fixture_and_test
    def test_neutral(self):
        """ a + neutral = a """
        return (self.neutral + self.one, self.one)

    @fixture_and_test
    def test_scalar_multiplication(self):
        """ an_int * a = a + ... + a (n times ) """
        left = self.scalar * self.one
        right = self.one.copy()
        for i in xrange(self.scalar - 1):
            right +=  self.one
        return (left, right)

    @fixture_and_test
    def test_multiplicative_identity(self):
        """ 1 * a = a """
        return 1 * self.one, self.one

    @fixture_and_test
    def test_associativity(self):
        """( a + b )  + c  = a + ( b + c )"""
        return (
            self.one + (self.other + self.another),
            (self.one + self.other) + self.another
        )

    @fixture_and_test
    def test_div_consisentcy(self):
        """ a * n  / 2  = a + ... + a n /2 times (n beign odd)"""
        right = (self.one * (self.scalar * 2)) / 2
        left = self.one
        for i in xrange(self.scalar - 1):
            left = left + self.one
        return left, right

    @fixture_and_test
    def test_fraction_consisentcy(self):
        """ a  / 2  = .5 * a"""
        ### not implemented because of the float equality shit
        return self.one / 2.0, .5 * self.one


    @fixture_and_test
    def test_multiply_scalar_symmetric(self):
        """ a * other_scalar = other_scalar * a """
        return (
           self.other_scalar * self.one,
           self.one * self.other_scalar
       )
    @fixture_and_test
    def test_multiply_scalar_associativity(self):
        """ ( an_int + other_scalar ) a = an_int * a  + other_scalar * a """
        return (
           (self.scalar + self.other_scalar) * self.one,
           self.scalar * self.one + self.other_scalar * self.one
       )
    @fixture_and_test
    def test_scalar_commutativity(self):
        """ an_int * a = a * an_int """
        return self.scalar * self.one, self.one * self.scalar

    @fixture_and_test
    def test_associativity_scalar_multiplication(self):
        """ ( an_int *  other_scalar ) * a = an_int ( other_scalar * a )"""
        return (
           (self.scalar * self.other_scalar) * self.one,
            self.scalar * (self.other_scalar * self.one))

    @fixture_and_test
    def test_negative(self):
        """ -a = -1 * a """
        return -1 * self.one, self.one.__neg__()

    @fixture_and_test
    def test_substraction(self):
        """ a - b = a + ( -1 * b) """
        return self.one - self.other, self.one + (-1 * self.other)

    @fixture_and_test
    def test_distributivity(self):
        """ an_int ( a + b) = an_int *a + an_int * b """
        return (
            self.scalar * (self.one + self.other),
            self.scalar * self.one + self.scalar * self.other
        )


    @fixture_and_test
    def conservation(self):
        """sum of the parts = sum of the total"""
        return (
            sum(self.collect_values(self.one + self.other)),
            sum(self.collect_values(self.one)) + \
                sum(self.collect_values(self.other))
       )

    @fixture_and_test
    def conservation_neg(self):
        """sum of the differences == differences of the sum"""
        return (
            sum(self.collect_values(self.one - self.other)),
            sum(self.collect_values(self.one)) - \
                sum(self.collect_values(self.other))
       )

    def LesserCommutativity(self):
        self.test_commutativity()
        self.test_neutral()

    def BetterCommutativity(self):
        self.test_multiplicative_identity()
        self.test_scalar_multiplication()
        self.test_scalar_commutativity()
        self.test_negative()
        self.test_associativity_scalar_multiplication()
        self.test_multiply_scalar_associativity()
        self.test_multiply_scalar_symmetric()
        self.test_associativity()
        self.test_substraction()
        self.test_distributivity()
        self.algebraic_logic = True

    def Conservation(self):
        self.conservation()
        self.conservation_neg()

if '__main__' == __name__:

    import os, sys, inspect
    cmd_folder = os.path.abspath(
        os.path.join(
            os.path.split(
                inspect.getfile( inspect.currentframe() )
            )[0] ,
            "."
        )
    )
    if cmd_folder not in sys.path:
       sys.path.insert(0, cmd_folder)

    try:
        from numpy import array as array

        ConsistentAlgebrae(
            neutral=array([0, 0, 0]),
            one=array([1, 2, 3]),
            another=array([5, 2, 3]),
            other=array([3, 4, -1]),
            equal=lambda left, right: (right == left).all(),
            )
    except Exception as e:
        print "only lamers dont use numpy"


    ConsistentAlgebrae(
        neutral=0,
        one=1,
        other=2,
        another=3

        )


    ConsistentAlgebrae(
        neutral=[],
        one=[1],
        other=[2],
        another=[42]
        )

    ConsistentAlgebrae(
        neutral="",
        one="1",
        other="2",
        another="4"
        )

    ConsistentAlgebrae(
        neutral=VectorDict(int, {}),
        one=VectorDict(int, {"one": 1, "one_and_two": 3}),
        other=VectorDict(int, {"one_and_two": - 1, "two": 2}),
        another=VectorDict(int, {"one": 3, 'two':  2, "three": 1}),
        collect_values=lambda x: x.values()
        )

    one = VectorDict(int, {"one": 1, "one_and_two": 12})
    other = VectorDict(int, {"one_and_two": - 9, "two": 2})

    print "just for fun \n\t%r\n\t+\n\t%r\n\t=\n\t%r" % (one, other, one + other)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#WTFPL
"""test for rules of consistency in algebrae"""
import os, sys, inspect
cmd_folder = os.path.abspath(
    os.path.join(
        os.path.split(
            inspect.getfile( inspect.currentframe() )
        )[0] ,
        ".."
    )
)
if cmd_folder not in sys.path:
   sys.path.insert(0, cmd_folder)



from VectorDict import VectorDict
from ConsistentAlgebrae import ConsistentAlgebrae

ConsistentAlgebrae(
    context="test",
    neutral=VectorDict(int, {}),
    one=VectorDict(int, {"one": 1, "one_and_two": 3}),
    other=VectorDict(int, {"one_and_two": - 1, "two": 2}),
    another=VectorDict(int, {"one": 3, 'two':  2, "three": 1}),
    collect_values=lambda x: x.values(),
    )





#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Dict behaving like a vector, and supporting all operations
the algebraic way
"""
import types
from Clause import Clause, is_leaf, is_container

from VectorDict import VectorDict, Path, tree_from_path
from collections import namedtuple, defaultdict
#WTFPL

__all__ = [ 'Coordinates', 'SparseMatrix' ]



Coordinates = namedtuple("Coordinates", "src, dst")

class SparseMatrix(VectorDict):
    """Sparse Matrice on dict of dict of dict
    a sparse matrix is a set of 
        - src path coordinate
        - dst path coordinate
        - and a fonction to apply on the src value 
        to transform it in destination value
   """

    def __init__(self, *coord_fonc, **kw  ):
        if coord_fonc and isinstance(  coord_fonc[0], types.TypeType) :
            super( VectorDict, self).__init__( *coord_fonc, **kw)
        else:
            super( VectorDict, self).__init__(  VectorDict, {} )
            for src, dst, funct in ( a_set  for a_set in coord_fonc):
                coord = Coordinates( src = src, dst = dst )
                self[coord] = funct

#    def copy(self):
#        return super( defaultdict, self).copy()

    def __call__(self, src):
        dst = VectorDict(VectorDict, {} )
        for coord, funct in self.iteritems():
            if not is_leaf(funct):
                real_func = lambda v : funct(v)
            else:
                real_func = funct
            
            dst.build_path(   
                *( list(coord.dst) + [ real_func( src.at( coord.src) ) ]) 
            )
        return dst

if '__main__' == __name__: 
    from VectorDict import convert_tree
    from Operation import  identity, mul, cast
    a = convert_tree( { 'a' : { 'b' : 1 , 'c' : 2 } , 'b' : 0 } )
    a.pprint()
    print a.at( [ 'a', 'c' ]  )
    m = SparseMatrix( 
        ( tuple( [ 'a', 'b' ] ), tuple([ 'mul', 'neg2' ] ), mul(-2) ),
        ( tuple([ 'a', 'c' ]), tuple([ 'mul', 'misplaced' ]), cast(float) ),
        ( tuple([ 'b' ])  , tuple([ 'a' ]) , lambda x : -4 ),
        ( tuple( [ 'a' ] ), tuple( [ 'a_dict' ] ), identity ),
        )
    m.pprint()
    print "**************"
    print m(a)
    m(a).pprint()
    print "iiiiiiiiiiiiiiiiiiiiiiiiiiiii" 
    w = SparseMatrix()
    w[ 
        Coordinates( src= tuple( [ ] ) , 
        dst =  tuple([ 'a_copy' ] ))
    ]=  identity 
    w[ 
        Coordinates( src= tuple( [ ] ) , 
        dst =  tuple([ 'inception' ] ))
    ]=  m.copy()  
   
    w(a).pprint()



from vector_dict.VectorDict import VectorDict, convert_tree, is_leaf
from vector_dict.Clause import *
def findme( v, a_tree):
    if not is_leaf(v): 
        return v.match_tree(a_tree)

positive = lambda v : v > 0

def pretty_present(list):
    print "Result "
    for el in list:
        print "path %r " % el[0]
        print "has value %s" %  (hasattr(el[1], 'tformat') and el[1].tformat() or el[1] )

    print
w = convert_tree( dict( 
    a = dict( c=  1),
    e = dict( d= 3.0) ,
    b = dict( c = 1 , d = 4 )
))

w.tprint()

pretty_present( w.find( lambda p, v : has_all( 'c', 'd' )(v) ) )

pretty_present( w.find( lambda p, v : findme(v, dict( 
                                        d= has_type(int), 
                                        c=has_type(int)   )
) ) ) 

pretty_present( w.find( lambda p, v : has_tree({  'a' : { 'c' : positive  } })(v)) )

pretty_present( w.find( lambda p, v : ( 
        has_type( int )(v) or has_type(float)(v)
    ) and  v > 3  ))
pretty_present( w.find( lambda p, v : p.endswith( [ 'c' ] ) ) )


"""
RESULTS : 
{
    a = {
        c = 1,
    },
    b = {
        c = 1,
        d = 4,
    },
    e = {
        d = 3.0,
    },
}
Result 
path ['b'] 
has value {
    c = 1,
    d = 4,
}

Result 
path ['b'] 
has value {
    c = 1,
    d = 4,
}

Result 
path [] 
has value {
    a = {
        c = 1,
    },
    b = {
        c = 1,
        d = 4,
    },
    e = {
        d = 3.0,
    },
}

Result 
path ['b', 'd'] 
has value 4

Result 
path ['a', 'c'] 
has value 1
path ['b', 'c'] 
has value 1
"""



import os
import sys
sys.path += [os.path.dirname( sys.modules[__name__].__file__) + os.sep + '..'  ]
print sys.path[-1]

import unittest
from VectorDict import VectorDict, convert_tree, Path, dot, cos
from Clause import is_leaf, has_type, is_container, anything
from Operation import mul, cast
from math import sqrt
class TestPath(unittest.TestCase):
    def setUp(self):
        self.a_path = Path( [ e for e in  xrange(10 ) ] )
    
    def test_build(self):
        self.assertEqual(
            self.a_path,
            ( 0, 1, 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9)
        )
    def test_startswith(self):
        self.assertEqual(
            self.a_path.startswith( 0, 1 , 2 ),
            True
        )

    def test_endswith(self):
        self.assertEqual(
            self.a_path.endswith( 8, 9),
            True
        )

    def test_contains1(self):
        self.assertEqual(
            self.a_path.contains( 2 , 3 , 4 ),
            True
        )

    def test_contains1(self):
        self.assertEqual(
            self.a_path.contains( 0, 1 , 2 ),
            True
        )
class TestVectorDict(unittest.TestCase):
    
    def setUp(self):
        self.easy = VectorDict( int, dict( x=1, y=1, z=0 ) )
        self.a_tree = dict( 
            a = 1, 
            b = dict( 
                c = 3.0, 
                d = dict( e = True)
            ), 
            point = self.easy 
        )
        self.cplx = convert_tree( self.a_tree )

    def test_convert(self):
        self.assertEqual( convert_tree( { 'a' : { 'b' : 1 } } ) ,
            VectorDict( VectorDict, 
                { 'a' : VectorDict( VectorDict, dict( b = 1 ) ) }
            )
        ) 

    
    def test_add_not_exists(self):
        self.easy += dict( a = 1 )
        self.assertEqual( self.easy['a'] , 1 )
    def test_add_exists(self):
        self.easy += dict( x = 1 )
        self.assertEqual( self.easy['x'] , 2 )
    def test_clause_has_type(self):
        self.assertEqual( 
            has_type(float)(3.0), 
            True)
    def test_is_leaf(self):
        """ we need a is_leaf more specialized must have a bug somewhre
        in find / match_tre
        TODO have one that tells is not vector_dict
        """
        self.assertEqual(
            all( [ is_leaf(e) for e in [ 
                set(), 
                frozenset(),
                list(),
                (e for e in range(0,10)),
                3.0,
                2,
                False,
                "ljlkjkl",
                ]]),
         True
         )
    def test_is_container(self):
        self.assertEqual(
            all( [ is_container(e) for e in [ 
                dict(), 
                set(), 
                frozenset(),
                list(),
                VectorDict(bool, {}),
                (e for e in range(0,10))
                ]]),
         True
         )
    def test_match_tree(self):
        self.assertEqual( 
            self.cplx["b"].match_tree( dict( c= 3.0 ,d = dict( e = True) )),
            True
        ) 
        
    def test_match_tree2(self):
        self.assertEqual( 
            self.cplx.match_tree( dict( 
                b=dict( c= 3.0, d = anything ))),
            True
        ) 
    def test_match_tree3(self):
        self.assertEqual( 
            self.cplx["b"].match_tree( 
                dict( 
                    c= has_type(float) ,
                    d = anything 
            )),
            True
        ) 
    def test_find1(self):
        self.assertEqual( 
            [ e for e in self.cplx.find( lambda p, v: ( 
                not is_leaf(v) and v.match_tree(
                dict( c= has_type(float) , d = is_container   )
                )
            ))  ][0][0], ('b',))

        
    def test_collision_build_path(self):
        self.assertRaises(
            ValueError,
            self.easy.build_path,  'z' , 1  )
    
            
    def test_collision_build_path2(self):
        self.assertRaises(
            ValueError,
            self.cplx.build_path,  'b' , 1  )
    
    def test_build_path(self):
        self.cplx.build_path(  'b' , "new",  "whatever"  )
        self.assertEqual( 
            self.cplx["b"]["new"],
            "whatever")
            
   # def test_add_path(self):
   #     self.cplx.add_path( [  'b' , "new",  "whatever"]  )
   #     self.cplx.add_path( [  'b' , "new",  "whatever"]  )
   #     self.assertEqual( 
   #         self.cplx["b"]["new"],
   #         "whatever" * 2 )
    
    def test_at1(self):
        self.assertRaises( 
            IndexError,
            self.cplx.at, [ 'a', 'b', 'c' ] )
    
    def test_at2_and_ope_mul(self):
        self.assertEqual( 
            self.cplx.at( [ 'b','c' ]) ,
            3.0
        )
    
    def test_at3_and_ope_mul(self):
        self.assertEqual( 
            self.cplx.at( [ 'b','c' ], mul(-1) ),
            -3.0
        )
    
    def test_at4(self):
        self.assertRaises( 
            Exception,
            self.cplx.at,  'a', 'b', 'c'  )
    
    def test_at5_copy(self):
        """not sure I ever used it"""
        self.assertIsNot( 
            self.cplx.at( [ 'b' ], None, True ),
            self.cplx["b"]
        )
    def test_get_at(self):
        self.assertIs(
            self.cplx.at( ['b', 'c' ] ),
            self.cplx.get_at( 'b' , 'c' )
    )


    def test_dot(self):
        self.assertEqual( 
            dot(
                VectorDict( int, dict( x=1 , y=1, z=0) ),
                VectorDict( int, dict( x=1 , y=1, z=1) ),
             ), 
             float( 2.0 )
        )

    def test_cos(self):
        self.assertAlmostEqual( 
            cos(
                VectorDict( int, dict( x=1 , y=0) ),
                VectorDict( int, dict( x=1 , y=1) ),
             ), 
             float( sqrt(2) / 2 )
        )
    
    def test_build_path_collision(self):
        self.easy.build_path( 'ext' , 'y',  1 )
        self.easy.build_path( 'ext' , 'z',  1 )
        self.assertEqual(
            len(self.easy['ext'].keys()), 
            2
        )
    
    def test_imul( self ):

        a_copy = self.easy.copy()
        self.easy *= -1
        self.assertEqual(
            self.easy['x'],
            -a_copy['x']
        )

    def test_bug_1( self ):
        """bug  #1 : * returns a copy where a reference is needed"""
        pt = self.easy
        pt *= -1
        self.assertEqual(
            self.easy,
            pt
        )

    def test_bug_2( self ):
        """bug #2 : self * other without affectation should not modify self.
        """
        self.easy *= -1
        a_copy = self.easy.copy()
        self.easy * a_copy
        self.assertEqual(
            self.easy,
            a_copy
        )

    def test_bug_3( self ):
        """bug  #3 : / returns a copy where a reference is needed"""
        self.easy.prune( 'z' )
        pt = self.easy.at([])
        pt /= -1
        self.assertEqual(
            self.easy,
            pt
        )


    def test_bug_4( self ):
        """bug #4 : self / other without affectation should not modify self.
        """
        self.easy.prune( 'z' )
        self.easy /= -1
        a_copy = self.easy.copy()
        self.easy / a_copy
        self.assertEqual(
            self.easy,
            a_copy
        )
    

    def test_bug_5( self ):
        """bug #5 : int * dict without affectation should not modify self.
        """
        a_copy = self.easy.copy()
        2 * self.easy
        self.assertEqual(
            self.easy,
            a_copy
        )
    
    def test_bug_6( self ):
        """bug #6 : int / dict without affectation should not modify self.
        """
        self.easy.prune( 'z' )
        a_copy = self.easy.copy()
        2.0 / self.easy
        self.assertEqual(
            self.easy,
            a_copy
        )
    def test_union_func_not_ok(self):
        a_lot_of_type= convert_tree( 
            dict( 
                a = "lkjlkjl",
                b = dict( 
                    e = set( [ 1 , 2 , 3 ] ),
                    f = lambda x ,y : 2 * y,
                
                ),
                p = dict(a  = "saint" )
            )
        )
        another_lot = convert_tree( 
            dict( a = "lkjlkjl",
                b = dict( 
                    e = set( [ 1 , 2 , 3 ] ),
                    f = lambda x ,y : 2 * y,
                    g = dict(),
                ),
                pp = dict(a  = "serre" )
            ),
        )
        cl = Exception('CollisionError','')
        
        self.assertRaises(
        Exception,
        a_lot_of_type.union, another_lot
        )
    def test_union_ok( self):
        a_lot_of_type= convert_tree( 
            dict( 
                a = "lkjlkjl",
                b = dict( 
                    e = set( [ 1 , 2 , 3 ] ),
                    f = 123.123123123,
                
                ),
                p = dict(a  = "saint" )
            )
        )
        another_lot = convert_tree( 
            dict( a = "lkjlkjl",
                b = dict( 
                    h = "fuck",
                    i = dict( a = 1 ),
                    e = set( [ 1 , 2 , 3, 5 ] ),
                    f = 123.123123123,
                    g = dict(),
                ),
                pp = dict(a  = "serre" )
            ),
        )
        un= a_lot_of_type.union(another_lot)
        expected = convert_tree( {
    'a' : 'lkjlkjl',
    'p' : {
        'a' : 'saint',
    },
    'pp' : {
        'a' : 'serre',
    },
    'b' : {
        'i' : {
            'a' : 1,
        },
        'e' : set([1, 2, 3, 5]),
        'g' : {
        },
        'f' : 123.123123123,
    },
})

        self.assertEqual( 
            un,
            expected) 
    
    def test_bug_convert_empty_mapping( self ):
        """bug #8 : empty mapping intialisation dont work"""
        bug = convert_tree( dict( a = {} ) )
        expected  = VectorDict(None, {'a': {}})
        
        self.assertEqual( 
            expected,
            bug

        )

if __name__ == '__main__':
    unittest.main(verbosity=2)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#WTFPL
"""maybe giving a sense to
dict( { a : 1 } ) + dict(  { a : 2 } ) + dict( { b : 2 } )
would be a practical for map reduce operations in nosql/large dataset
(HDF5 iterators) like context. Je dis ça, je dis rien"""
import os, sys, inspect
cmd_folder = os.path.abspath(
    os.path.join(
        os.path.split(
            inspect.getfile( inspect.currentframe() )
        )[0] ,
        ".."
    )
)
if cmd_folder not in sys.path:
   sys.path.insert(0, cmd_folder)


from vector_dict.VectorDict import iter_object,  VectorDict
from csv import reader, writer
import os
from io import StringIO
from numpy import array

### CSV of langage users in a nosql DB (or any cloudish crap)
### having individual votes of cooolness per langage
### should be a very large dataset to be funnier

mocking_nosql = u"""george,FR,fun,perl,2
roger,FR,serious,python,3
christine,DE,fun,python,3
bob,US,serious,php,1
isabelle,FR,fun,perl,10
Kallista,FR,unfun,c#,-10
Nellev,FR,typorigid,python,1
haypo,FR,javabien,python,1
potrou,FR,globally locked,python,1
petra,DE,sexy,VHDL,69"""

nosql_iterator = reader

### interesting columns/key we want to emit
COUNTRY = 1
LANGAGE = 3
COOLNESS = 4

w = writer(os.sys.stdout)

## basic example for counting langage users nothing more interesting than
## what collections.counter does
print ""
w.writerow(["langage", "how many users"])
map(w.writerow,
    iter_object(
        reduce(
            ## well that is a complex reduce operation :)
            VectorDict.__iadd__,
            map(
                lambda document: VectorDict(int, {document[LANGAGE]: 1}),
                nosql_iterator(StringIO(mocking_nosql))
            )
        ),
        flatten = True
    )
)
"""expected result :
langage,how many users
python,3
c#,1
php,1
VHDL,1
perl,2
"""
print "\n" * 2 + "next test\n"
### example with group by + aggregation of a counter
### counting all langage per country with their coolness and number of users
### Hum .... I miss a sort and a  limit by to be completely SQL like compatible
w.writerow(["country", "langage", "coolness", "howmany"])
map(w.writerow,
    iter_object(
        ##nosql like reduce
        reduce(
            ## well that is the same very complex reduce operation :)
            VectorDict.__iadd__,
            ##nosql like map where we emit interesting subset of the record
            map(
                lambda document: VectorDict(
                    VectorDict, {
                        #KEY
                        document[COUNTRY]:
                        VectorDict(
                            array,
                            {
                                #GROUPBY
                                document[LANGAGE]:
                                #AGGREGATOR
                                array([
                                    #Counter
                                    int(document[COOLNESS]),
                                    #presence
                                    1
                                ])
                            }
                        ),
                        ## making a (sub) total on the fly
                        'total_coolness_and_voters': array([
                            int(document[COOLNESS]), 1
                        ])
                    }
                ),

                ## nosql like filter that should be in the map if it was nosql
                ## maybe we also need a map that accepts None as a result and
                ## skip the result in the resulting iterator,
                ## or a skip() callable in lambda ?
                ## or sub()  like in perl with curly braces
                ## joke : combining map / filter is easy enough
                filter(
                    lambda document: "php" != document[LANGAGE],
                    nosql_iterator(StringIO(mocking_nosql))
                )
            )
        ),
            flatten = True
    )
)
"""expected result :
country,langage,coolness,howmany
FR,python,4,2
FR,c#,-10,1
FR,perl,12,2
total_votes_and_coolness,7,78
DE,python,3,1
DE,VHDL,69,1
"""



#!/usr/bin/env pyrhon
# -*- coding: "utf-8" -*-

from distutils.command.build_py import build_py as _build_py
from distutils.core import setup

def test():
    """Specialized Python source builder."""

    from vector_dict.VectorDict import VectorDict
    from  vector_dict.ConsistentAlgebrae import ConsistentAlgebrae

    ConsistentAlgebrae(
        context="test",
        neutral=VectorDict(int, {}),
        one=VectorDict(int, {"one": 1, "one_and_two": 3}),
        other=VectorDict(int, {"one_and_two": - 1, "two": 2}),
        another=VectorDict(int, {"one": 3, 'two':  2, "three": 1}),
        collect_values=lambda x: x.values()
        )

test()

setup(
        name='VectorDict',
        version='0.6.0',
        author='Julien Tayon',
        author_email='julien@tayon.net',
        packages=['vector_dict'],
        url='http://readthedocs.org/docs/vectordict/en/latest/',
        license='LICENSE.txt',
        test_suite='vector_dict.test.test_vector',
        description='dict with recursive operations (algebraic, set, logicial)',
        requires=[
        "collections","math"
        ],
        classifiers=[
          'Development Status :: 4 - Beta',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: Python Software Foundation License',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: Microsoft :: Windows',
          'Operating System :: POSIX',
          'Programming Language :: Python',
          ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
.. module:: yep
"""


from vector_dict.VectorDict import VectorDict, flattening, is_generator 
def find():
    """
    How to find all the sub dict in a tree that have the keys x, y, z at the same time time"""


    v = VectorDict( float, dict( x = 1, y = 1, z = 2 ) )
    w = VectorDict(float, dict( w = v, z = v ) )
    e = VectorDict(float, dict( a = w ) )
    print "%r" % [ el for el in e.find( 
            lambda k, v : 
                isinstance(v, dict) and set( ['x', 'y', 'z'] ) == set( v.keys() )
        ) ] 
    e.pprint()
find()
